@@ -323,6 +323,14 @@ static void DecodeHeader(guchar *Data, gint Bytes,
 	
 	State->HeaderSize+=I;
 	
+	if (State->HeaderSize < 0) {
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+			     _("Invalid header in icon"));
+		return;
+	}
+
  	if (State->HeaderSize>State->BytesInHeaderBuf) {
 	        guchar *tmp=g_try_realloc(State->HeaderBuf,State->HeaderSize);
 		if (!tmp) {
@@ -942,7 +950,7 @@ fill_entry (IconEntry *icon,
 	
 	if (hot_x > -1 && hot_y > -1) {
 		icon->hot_x = hot_x;
-		icon->hot_x = hot_y;
+		icon->hot_y = hot_y;
 		if (icon->hot_x >= icon->width || icon->hot_y >= icon->height) {
 			g_set_error (error,
 				     GDK_PIXBUF_ERROR,
@@ -1202,7 +1210,7 @@ MODULE_ENTRY (ico, fill_info) (GdkPixbufFormat *info)
 	info->description = N_("The ICO image format");
 	info->mime_types = mime_types;
 	info->extensions = extensions;
-	info->flags = GDK_PIXBUF_FORMAT_WRITABLE;
+	info->flags = GDK_PIXBUF_FORMAT_WRITABLE | GDK_PIXBUF_FORMAT_THREADSAFE;
 	info->license = "LGPL";
 }
 
