@@ -80,16 +80,20 @@ struct _GtkActionClass
   void (*_gtk_reserved4) (void);
 };
 
-GType        gtk_action_get_type               (void);
+GType        gtk_action_get_type               (void) G_GNUC_CONST;
 GtkAction   *gtk_action_new                    (const gchar *name,
 						const gchar *label,
 						const gchar *tooltip,
 						const gchar *stock_id);
-const gchar* gtk_action_get_name               (GtkAction     *action);
+G_CONST_RETURN gchar* gtk_action_get_name      (GtkAction     *action);
 gboolean     gtk_action_is_sensitive           (GtkAction     *action);
 gboolean     gtk_action_get_sensitive          (GtkAction     *action);
+void         gtk_action_set_sensitive          (GtkAction     *action,
+						gboolean       sensitive);
 gboolean     gtk_action_is_visible             (GtkAction     *action);
 gboolean     gtk_action_get_visible            (GtkAction     *action);
+void         gtk_action_set_visible            (GtkAction     *action,
+						gboolean       visible);
 void         gtk_action_activate               (GtkAction     *action);
 GtkWidget*   gtk_action_create_icon            (GtkAction     *action,
 						GtkIconSize    icon_size);
@@ -102,6 +106,7 @@ void         gtk_action_disconnect_proxy       (GtkAction     *action,
 GSList*      gtk_action_get_proxies            (GtkAction     *action);
 void         gtk_action_connect_accelerator    (GtkAction     *action);
 void         gtk_action_disconnect_accelerator (GtkAction     *action);
+G_CONST_RETURN gchar *gtk_action_get_accel_path (GtkAction     *action);
 
 /* protected ... for use by child actions */
 void         gtk_action_block_activate_from    (GtkAction     *action,
