@@ -25,7 +25,6 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkbutton.h"
 #include "gtkdialog.h"
 #include "gtkhbbox.h"
@@ -37,6 +36,7 @@
 #include "gtkmain.h"
 #include "gtkintl.h"
 #include "gtkbindings.h"
+#include "gtkalias.h"
 
 #define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_DIALOG, GtkDialogPrivate))
 
@@ -359,15 +359,15 @@ gtk_dialog_map (GtkWidget *widget)
       while (GTK_IS_LABEL (window->focus_widget));
 
       tmp_list = children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
-
+      
       while (tmp_list)
 	{
 	  GtkWidget *child = tmp_list->data;
-
-	  if (window->focus_widget == NULL ||
-	      (child == window->focus_widget && 
-	       child != window->default_widget && 
-	       window->default_widget))
+	  
+	  if ((window->focus_widget == NULL || 
+	       child == window->focus_widget) && 
+	      child != window->default_widget &&
+	      window->default_widget)
 	    {
 	      gtk_widget_grab_focus (window->default_widget);
 	      break;
@@ -375,7 +375,7 @@ gtk_dialog_map (GtkWidget *widget)
 	  
 	  tmp_list = tmp_list->next;
 	}
-
+      
       g_list_free (children);
     }
 }
@@ -552,18 +552,9 @@ get_response_data (GtkWidget *widget,
 static void
 action_widget_activated (GtkWidget *widget, GtkDialog *dialog)
 {
-  ResponseData *ad;
   gint response_id;
   
-  g_return_if_fail (GTK_IS_DIALOG (dialog));
-
-  response_id = GTK_RESPONSE_NONE;
-  
-  ad = get_response_data (widget, TRUE);
-
-  g_assert (ad != NULL);
-  
-  response_id = ad->response_id;
+  response_id = _gtk_dialog_get_response_for_widget (dialog, widget);
 
   gtk_dialog_response (dialog, response_id);
 }
@@ -587,7 +578,7 @@ gtk_dialog_add_action_widget (GtkDialog *dialog,
                               gint       response_id)
 {
   ResponseData *ad;
-  gint signal_id = 0;
+  guint signal_id;
   
   g_return_if_fail (GTK_IS_DIALOG (dialog));
   g_return_if_fail (GTK_IS_WIDGET (child));
@@ -599,7 +590,7 @@ gtk_dialog_add_action_widget (GtkDialog *dialog,
   if (GTK_IS_BUTTON (child))
     signal_id = g_signal_lookup ("clicked", GTK_TYPE_BUTTON);
   else
-    signal_id = GTK_WIDGET_GET_CLASS (child)->activate_signal != 0;
+    signal_id = GTK_WIDGET_GET_CLASS (child)->activate_signal;
 
   if (signal_id)
     {
@@ -741,8 +732,7 @@ gtk_dialog_set_response_sensitive (GtkDialog *dialog,
   while (tmp_list != NULL)
     {
       GtkWidget *widget = tmp_list->data;
-      ResponseData *rd = g_object_get_data (G_OBJECT (widget),
-                                            "gtk-dialog-response-data");
+      ResponseData *rd = get_response_data (widget, FALSE);
 
       if (rd && rd->response_id == response_id)
         gtk_widget_set_sensitive (widget, setting);
@@ -777,8 +767,7 @@ gtk_dialog_set_default_response (GtkDialog *dialog,
   while (tmp_list != NULL)
     {
       GtkWidget *widget = tmp_list->data;
-      ResponseData *rd = g_object_get_data (G_OBJECT (widget),
-                                            "gtk-dialog-response-data");
+      ResponseData *rd = get_response_data (widget, FALSE);
 
       if (rd && rd->response_id == response_id)
 	gtk_widget_grab_default (widget);
@@ -833,7 +822,7 @@ gtk_dialog_set_has_separator (GtkDialog *dialog,
       dialog->separator = NULL;
     }
 
-  g_object_notify (G_OBJECT (dialog), "has_separator");
+  g_object_notify (G_OBJECT (dialog), "has-separator");
 }
 
 /**
@@ -984,7 +973,7 @@ run_destroy_handler (GtkDialog *dialog, gpointer data)
 gint
 gtk_dialog_run (GtkDialog *dialog)
 {
-  RunInfo ri = { NULL, GTK_RESPONSE_NONE, NULL };
+  RunInfo ri = { NULL, GTK_RESPONSE_NONE, NULL, FALSE };
   gboolean was_modal;
   gulong response_handler;
   gulong unmap_handler;
@@ -1035,7 +1024,6 @@ gtk_dialog_run (GtkDialog *dialog)
   g_main_loop_unref (ri.loop);
 
   ri.loop = NULL;
-  ri.destroyed = FALSE;
   
   if (!ri.destroyed)
     {
@@ -1237,3 +1225,5 @@ gtk_dialog_set_alternative_button_order_from_array (GtkDialog *dialog,
     }
 }
 
+#define __GTK_DIALOG_C__
+#include "gtkaliasdef.c"
