@@ -34,13 +34,13 @@
 #include <string.h>
 #include <locale.h>
 #include "gdk/gdkkeysyms.h"
-#include "gtkalias.h"
 #include "gtkbindings.h"
 #include "gtkspinbutton.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtksettings.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 #define MIN_SPIN_BUTTON_WIDTH              30
 #define SPIN_BUTTON_INITIAL_TIMER_DELAY    200
@@ -520,6 +520,8 @@ gtk_spin_button_unmap (GtkWidget *widget)
 {
   if (GTK_WIDGET_MAPPED (widget))
     {
+      gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (widget));
+
       gdk_window_hide (GTK_SPIN_BUTTON (widget)->panel);
       GTK_WIDGET_CLASS (parent_class)->unmap (widget);
     }
@@ -617,40 +619,6 @@ compute_double_length (double val, int digits)
   return a + digits + extra;
 }
 
-/* Keep in sync with gtkentry.c !
- */
-static void
-get_borders (GtkEntry *entry,
-             gint     *xborder,
-             gint     *yborder)
-{
-  GtkWidget *widget = GTK_WIDGET (entry);
-  gint focus_width;
-  gboolean interior_focus;
-
-  gtk_widget_style_get (widget,
-			"interior-focus", &interior_focus,
-			"focus-line-width", &focus_width,
-			NULL);
-
-  if (entry->has_frame)
-    {
-      *xborder = widget->style->xthickness;
-      *yborder = widget->style->ythickness;
-    }
-  else
-    {
-      *xborder = 0;
-      *yborder = 0;
-    }
-
-  if (!interior_focus)
-    {
-      *xborder += focus_width;
-      *yborder += focus_width;
-    }
-}
-
 static void
 gtk_spin_button_size_request (GtkWidget      *widget,
 			      GtkRequisition *requisition)
@@ -709,7 +677,7 @@ gtk_spin_button_size_request (GtkWidget      *widget,
       w = PANGO_PIXELS (MIN (string_len, max_string_len) * digit_width);
       width = MAX (width, w);
       
-      get_borders (entry, &xborder, &yborder);
+      _gtk_entry_get_borders (entry, &xborder, &yborder);
       
       xborder += 2; /* INNER_BORDER */
 
@@ -1594,7 +1562,7 @@ gtk_spin_button_configure (GtkSpinButton  *spin_button,
   if (spin_button->climb_rate != climb_rate)
     {
       spin_button->climb_rate = climb_rate;
-      g_object_notify (G_OBJECT (spin_button), "climb_rate");
+      g_object_notify (G_OBJECT (spin_button), "climb-rate");
     }
   g_object_thaw_notify (G_OBJECT (spin_button));
 
@@ -1629,6 +1597,10 @@ gtk_spin_button_new (GtkAdjustment *adjustment,
  * initially set to the minimum value and a page increment of 10 * @step
  * is the default. The precision of the spin button is equivalent to the 
  * precision of @step.
+ *
+ * Note that the way in which the precision is derived works best if @step 
+ * is a power of ten. If the resulting precision is not suitable for your 
+ * needs, use gtk_spin_button_set_digits() to correct it. 
  * 
  * Return value: The new spin button as a #GtkWidget.
  **/
@@ -1641,7 +1613,7 @@ gtk_spin_button_new_with_range (gdouble min,
   GtkSpinButton *spin;
   gint digits;
 
-  g_return_val_if_fail (min < max, NULL);
+  g_return_val_if_fail (min <= max, NULL);
   g_return_val_if_fail (step != 0.0, NULL);
 
   spin = g_object_new (GTK_TYPE_SPIN_BUTTON, NULL);
@@ -1952,7 +1924,7 @@ gtk_spin_button_set_update_policy (GtkSpinButton             *spin_button,
   if (spin_button->update_policy != policy)
     {
       spin_button->update_policy = policy;
-      g_object_notify (G_OBJECT (spin_button), "update_policy");
+      g_object_notify (G_OBJECT (spin_button), "update-policy");
     }
 }
 
@@ -2109,7 +2081,7 @@ gtk_spin_button_set_snap_to_ticks (GtkSpinButton *spin_button,
       if (new_val && GTK_ENTRY (spin_button)->editable)
 	gtk_spin_button_update (spin_button);
       
-      g_object_notify (G_OBJECT (spin_button), "snap_to_ticks");
+      g_object_notify (G_OBJECT (spin_button), "snap-to-ticks");
     }
 }
 
@@ -2285,3 +2257,6 @@ spin_button_redraw (GtkSpinButton *spin_button)
       gdk_window_invalidate_rect (spin_button->panel, NULL, TRUE);
     }
 }        
+
+#define __GTK_SPIN_BUTTON_C__
+#include "gtkaliasdef.c"
